/*
 * $Id: CookieSnatch.java,v 1.2 2003/10/13 21:58:32 enigma Exp $
 * Created on Oct 13, 2003
 */
package org.ninjasoft.cookiesnatch;

import java.net.*;
import java.io.*;
import java.util.*;

public class CookieSnatch {
    /** Should debugging statements be enabled? */
    public static final boolean DEBUG = true;
    
    /** These are the cookies that have been given to us by the server */
    private Vector cookies = new Vector();
    
    public CookieSnatch() {}
    
    public boolean doAction(String url, String method, String data, boolean showResults) 
    throws IOException {
        String key; 
        String value;
        int counter = 0;
        URL connectUrl = new URL(url);
        HttpURLConnection connection = (HttpURLConnection) connectUrl.openConnection();
        // Set to GET or POST
        if (method.equalsIgnoreCase("POST") && (data != null)) {
            connection.setDoOutput(true);
        }
        connection.setRequestMethod(method.toUpperCase());
        // Set cookies
        for (Iterator i = cookies.iterator(); i.hasNext(); )
            connection.setRequestProperty("Cookie", i.next().toString());
        // Send post data
        if (method.equalsIgnoreCase("POST") && (data != null)) {
            OutputStream out = connection.getOutputStream();
            out.write(data.getBytes());
        }
        // Check response code
        if (connection.getResponseCode() != 200)
            return false;
        // Check the cookies we got
        println("##### Got the following cookies:");
        for (value = connection.getHeaderField(counter); (value = connection.getHeaderField(counter)) != null; counter++) {
            key = connection.getHeaderFieldKey(counter);
            println(key + " == " + value);
            if ((key != null) && (key.equalsIgnoreCase("Set-Cookie")))
                cookies.add(value);
        }
        if (showResults) {
            StringBuffer page = new StringBuffer();
            InputStream in = connection.getInputStream();
            byte buff[] = new byte[1024];
            int len = 0;
            while ((len = in.read(buff)) > 0)
                for (int i=0; i<len; i++)
                    page.append((char) buff[i]);
            println("##### Got the following content:");
            println(stripTags(page.toString()));
        }
        return true;
    }
    
    public void writeCookies(String base) 
    throws IOException {
        FileOutputStream out1 = new FileOutputStream(new File("grabbed_cookies.txt"), false);
        FileOutputStream out2 = new FileOutputStream(new File("cookies.txt"), false);
        for (Iterator i = cookies.iterator(); i.hasNext(); ) {
            String cookie = i.next().toString();
            int pos1 = cookie.indexOf('=');
            int pos2 = cookie.indexOf(';');
            String name = cookie.substring(0, pos1);
            String value = cookie.substring(pos1+1, pos2);
            println("##### Writing cookie to file: " + cookie);
            // Write "HTTP header" formatted file
            out1.write("Cookie: ".getBytes());
            out1.write(cookie.getBytes());
            out1.write("\n".getBytes());
            // Write "Netscape cookies.txt" formatted file
            out2.write(base.getBytes());
            out2.write('\t');
            out2.write("TRUE\t/\tFALSE\t2145801432\t".getBytes());
            out2.write(name.getBytes());
            out2.write('\t');
            out2.write(value.getBytes());
            out2.write("\n".getBytes());
        }
        out1.close();
        out2.close();
    }
    
    /**
     * Ultra-simplistic function to strip HTML tags from a string...used to
     * simplify output.  (Note: formatting tags for newlines like <BR> and
     * <P>...</P> are removed, too)
     * @param s
     * @return
     */
    public static final String stripTags(String s) {
        StringBuffer result = new StringBuffer();
        boolean copying = true;
        int max = s.length();
        for (int i=0; i<max; i++) {
            char c = s.charAt(i);
            if (copying && (c == '<'))
                copying = false;
            else if (!copying && (c == '>'))
                copying = true;
            else if (copying) {
                if ((c != '\r') && (c != '\n'))
                    result.append(c);
            }
        }
        return result.toString();
    }
    
    /**
     * A debug println...basically only print if the DEBUG flag is enabled
     * @param s
     */
    public static final void println(String s) {
        if (DEBUG)
            System.out.println(s);
    }
    
    
    public static void main(String[] argv) throws Exception {
        CookieSnatch cs = new CookieSnatch();
        println("Starting");
        Properties props = new Properties();
        props.load(new FileInputStream("site.properties"));
        // Get initial session cookie
        cs.doAction(props.getProperty("org.ninjasoft.cookiesnatch.firsturl", ""), 
                    props.getProperty("org.ninjasoft.cookiesnatch.firsturl.method", "GET"), 
                    props.getProperty("org.ninjasoft.cookiesnatch.firsturl.postdata"), 
                    true);
        // Log in to form using that cookie
        cs.doAction(props.getProperty("org.ninjasoft.cookiesnatch.secondurl", ""), 
                    props.getProperty("org.ninjasoft.cookiesnatch.secondurl.method", "GET"), 
                    props.getProperty("org.ninjasoft.cookiesnatch.secondurl.postdata"), 
                    true);
        cs.writeCookies(props.getProperty("org.ninjasoft.cookiesnatch.cookiebase"));
        println("Ending");
    }
}
